/*
 * case8.cpp
 *
 *  Created on: Jan 24, 2024
 *      Author: Dario
 */

/**
 * A thread gets stuck in an infinite loop and the watchdog restarts it. 
 */
#include <sta/tasty/config.hpp>
#ifdef TASTY_CASE_8

#include <sta/tacos.hpp>
#include <sta/tasty/utils.hpp>
#include <sta/tacos/watchdog.hpp>
#include <sta/rtos/mutex.hpp>

namespace sta
{
    namespace tasty
    {
        static RtosMutex * mutexA;
        static RtosMutex * mutexB;

        class ThreadA : public tacos::TacosThread
        {
        public:
            ThreadA()
                : tacos::TacosThread{"A", osPriorityNormal}
            {

            }

            void func() override
            {
                mutexA->acquire();

                this->sleep(10);

                mutexB->acquire();
            }
        };

        class ThreadB : public tacos::TacosThread
        {
        public:
            ThreadB()
                : tacos::TacosThread{"B", osPriorityNormal}
            {

            }

            void func() override
            {
                mutexB->acquire();

                this->sleep(10);

                mutexA->acquire();
            }
        };

        class Observer : public tacos::TacosThread
        {
        public:
            Observer()
                : tacos::TacosThread{"Observer", osPriorityNormal}
            {

            }

            void func() override
            {
                this->sleep(2000);

                STA_TASTY_ASSERT(tacos::Watchdog::instance()->getNumRestarts() == 2);

                STA_TASTY_TERMINATE();
            }
        };

        void onTastyInit()
        {
            mutexA = new RtosMutex("MutA");
            mutexB = new RtosMutex("MutB");

            tacos::addThread<ThreadA>({0});
            tacos::addThread<ThreadB>({0});
            tacos::addThread<Observer>({0});
        }
    } // namespace tasty
}

#endif // TASTY_CASE_8


